local _, ns = ...
local OnCast = ns.addon
local Util = OnCast:NewModule("Util")

-- Lua APIs
local pairs = pairs
local typeof = type
local tonumber = tonumber
local sformat = string.format
local supper = string.upper
local slower = string.lower
local sgmatch = string.gmatch
local sgsub = string.gsub
local tconcat = table.concat

local function deepCopyExclude(original, exclude)
    local copy = {}
    for k, v in pairs(original) do
        if not exclude[k] then
            if typeof(v) == "table" then
                v = deepCopyExclude(v, exclude)
            end
            copy[k] = v
        end
    end
    return copy
end
local function deepCopy(original)
    local copy = {}
    for k, v in pairs(original) do
        if typeof(v) == "table" then
            v = deepCopy(v)
        end
        copy[k] = v
    end
    return copy
end

function Util.tableDeepCopy(original, exclude)
    if original == nil then return nil end
    if exclude then
        return deepCopyExclude(original, exclude)
    end
    return deepCopy(original)
end

function Util.getTableSize(table, exclude)
    local count = 0
    if table then
        for k in pairs(table) do
            if not (exclude and exclude[k]) then
                count = count + 1
            end
        end
    end
    return count
end

function Util.concatTableKeys(table, separator, formatKeyFunc)
    local keys = {}
    if formatKeyFunc then
        for k in pairs(table) do
            keys[#keys + 1] = formatKeyFunc(k)
        end
    else
        for k in pairs(table) do
            keys[#keys + 1] = k
        end
    end
    return tconcat(keys, separator)
end

-- function Util.sortStringArray(string_array)
--     table.sort(string_array, function(a, b)
--         return supper(a) < supper(b)
--     end)
-- end

local SPLIT_PATTERNS = {}
function Util.addSplitPattern(separator)
    SPLIT_PATTERNS[separator] = "([^"..separator.."]+)" -- Match all characters between separators
end

function Util.stringSplitEF(input, separator, array)
    if not input then return 0 end
    local n = 0
    for value in sgmatch(input, SPLIT_PATTERNS[separator]) do
        n = n + 1
        array[n] = value
    end
    return n
end

function Util.stringSplit(input, separator)
    if not input then return nil, 0 end
    local list = {}
    local n = 0
    local pattern = SPLIT_PATTERNS[separator] or "([^"..separator.."]+)" -- Match all characters between separators
    for value in sgmatch(input, pattern) do 
        n = n + 1
        list[n] = value
    end
    return list, n
end

function Util.stringTrim(value, start_patten, end_patten)
    start_patten = start_patten or "%s*" -- Default to whitespace characters
    end_patten = end_patten or "%s*"
    value = sgsub(value, "^"..start_patten.."(.-)"..end_patten.."$", "%1")
    return value
end

function Util.stringTitleCase(input)
    local function tchelper(first, rest)
        return supper(first)..slower(rest)
    end
    return sgsub(input, "(%a)([%w_']*)", tchelper)
end

-- Colors
function Util.colorStringToTable(color)
    local n = 0
    local colors = {}
    sgsub(color, "([0-9,a-f,A-F][0-9,a-f,A-F])", function(v)
        n = n + 1
        colors[n] = v
    end)
    return n, colors
end

function Util.colorTableToString(colors)
    return tconcat(colors)
end

function Util.hexToInt(hex) 
    local v = sformat("%d", "0x"..hex)
    return tonumber(v)
end

function Util.intToHex(int)
    return sformat("%02x", int)
end
